@@ -365,7 +365,7 @@ function RerootFolder(uri, newFolder, viewType, viewFlags, sortType, sortOrder)
 
   SetUpToolbarButtons(uri);
 
-  UpdateStatusMessageCounts(newFolder);
+  UpdateStatusMessageCounts(gMsgFolderSelected);
   
   UpdateMailToolbar("reroot folder in 3 pane");
   // hook for extra toolbar items
@@ -681,6 +681,9 @@ function CreateBareDBView(originalView, msgFolder, viewType, viewFlags, sortType
       case nsMsgViewType.eShowQuickSearchResults:
           dbviewContractId += "quicksearch";
           break;
+      case nsMsgViewType.eSearch:
+          dbviewContractId += "search";
+          break;
       case nsMsgViewType.eShowThreadsWithUnread:
           dbviewContractId += "threadswithunread";
           break;
@@ -867,9 +870,12 @@ function FolderPaneSelectionChange()
                 var msgDatabase = msgFolder.getMsgDatabase(msgWindow);
                 if (msgDatabase)
                 {
+                  gSearchSession = null;
                   var dbFolderInfo = msgDatabase.dBFolderInfo;
                   sortType = dbFolderInfo.sortType;
                   sortOrder = dbFolderInfo.sortOrder;
+                  viewType = dbFolderInfo.viewType;
+                  viewFlags = dbFolderInfo.viewFlags;
                   if (folderFlags & MSG_FOLDER_FLAG_VIRTUAL)
                   {
                     viewType = nsMsgViewType.eShowQuickSearchResults;
@@ -894,7 +900,6 @@ function FolderPaneSelectionChange()
                     }
                     else
                     {
-                      gSearchSession = null;
                       uriToLoad = srchFolderUri;
                       // we need to load the db for the actual folder so that many hdrs to download
                       // will return false...
@@ -907,12 +912,6 @@ function FolderPaneSelectionChange()
 //                      gSearchInput.showingSearchCriteria = false;
                     }
                   }
-                  else
-                  {
-                    gSearchSession = null;
-                    viewFlags = dbFolderInfo.viewFlags;
-                    viewType = dbFolderInfo.viewType;
-                  }
                   msgDatabase = null;
                   dbFolderInfo = null;
                 }
