@@ -1,11 +1,11 @@
 /* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* ***** BEGIN LICENSE BLOCK *****
- * Version: NPL 1.1/GPL 2.0/LGPL 2.1
+ * Version: MPL 1.1/GPL 2.0/LGPL 2.1
  *
- * The contents of this file are subject to the Netscape Public License
- * Version 1.1 (the "License"); you may not use this file except in
- * compliance with the License. You may obtain a copy of the License at
- * http://www.mozilla.org/NPL/
+ * The contents of this file are subject to the Mozilla Public License Version
+ * 1.1 (the "License"); you may not use this file except in compliance with
+ * the License. You may obtain a copy of the License at
+ * http://www.mozilla.org/MPL/
  *
  * Software distributed under the License is distributed on an "AS IS" basis,
  * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
@@ -14,7 +14,7 @@
  *
  * The Original Code is Mozilla Communicator client code.
  *
- * The Initial Developer of the Original Code is 
+ * The Initial Developer of the Original Code is
  * Netscape Communications Corporation.
  * Portions created by the Initial Developer are Copyright (C) 1998
  * the Initial Developer. All Rights Reserved.
@@ -24,16 +24,16 @@
  *   Dean Tessman <dean_tessman@hotmail.com>
  *
  * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or 
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
  * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
  * in which case the provisions of the GPL or the LGPL are applicable instead
  * of those above. If you wish to allow use of your version of this file only
  * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the NPL, indicate your
+ * use your version of this file under the terms of the MPL, indicate your
  * decision by deleting the provisions above and replace them with the notice
  * and other provisions required by the GPL or the LGPL. If you do not delete
  * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the NPL, the GPL or the LGPL.
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
 
@@ -54,13 +54,15 @@
 #include "nsIDOMEventTarget.h"
 #include "nsIDOMKeyEvent.h"
 #include "nsIPrivateDOMEvent.h"
+#include "nsIDOMCompositionListener.h"
 #include "nsIDOMDocument.h"
 #include "nsIDOMElement.h"
 #include "nsIDOMNSHTMLInputElement.h"
+#include "nsIDocument.h"
 #include "nsIScriptGlobalObject.h"
 #include "nsIContent.h"
 #include "nsIPresShell.h"
-#include "nsIPresContext.h"
+#include "nsPresContext.h"
 #include "nsIView.h"
 #include "nsIFrame.h"
 #include "nsIWidget.h"
@@ -79,6 +81,7 @@ NS_INTERFACE_MAP_BEGIN(nsFormFillController)
   NS_INTERFACE_MAP_ENTRY(nsIDOMFormListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMMouseListener)
   NS_INTERFACE_MAP_ENTRY(nsIDOMLoadListener)
+  NS_INTERFACE_MAP_ENTRY(nsIDOMCompositionListener)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsISupports, nsIFormFillController)
   NS_INTERFACE_MAP_ENTRY_AMBIGUOUS(nsIDOMEventListener, nsIDOMFocusListener)
 NS_INTERFACE_MAP_END
@@ -117,10 +120,10 @@ nsFormFillController::~nsFormFillController()
 
 ////////////////////////////////////////////////////////////////////////
 
-nsRect&
+nsRect
 GetScreenOrigin(nsIDOMElement* aElement)
 {
-  nsRect* rect = new nsRect(0,0,0,0);
+  nsRect rect(0,0,0,0);
   nsSize size;
  
   nsCOMPtr<nsIContent> content = do_QueryInterface(aElement);
@@ -131,8 +134,7 @@ GetScreenOrigin(nsIDOMElement* aElement)
     nsIPresShell* presShell = doc->GetShellAt(0);
     
     if (presShell) {
-      nsCOMPtr<nsIPresContext> presContext;
-      presShell->GetPresContext(getter_AddRefs(presContext));
+      nsPresContext* presContext = presShell->GetPresContext();
 
       if (presContext) {
         // Get the scale from that Presentation Context
@@ -144,27 +146,27 @@ GetScreenOrigin(nsIDOMElement* aElement)
 
         nsIView* view;
         nsPoint offset;
-        frame->GetOffsetFromView(presContext, offset, &view);
+        frame->GetOffsetFromView(offset, &view);
         if (view) {
           nsPoint widgetOffset(0, 0);
           nsIWidget* widget = view->GetNearestWidget(&widgetOffset);
           if (widget) {
             nsRect oldBox(0,0,0,0);
-            widget->WidgetToScreen(oldBox, *rect);
+            widget->WidgetToScreen(oldBox, rect);
           }
           
-          rect->x += NSTwipsToIntPixels(offset.x+widgetOffset.x, scale);
-          rect->y += NSTwipsToIntPixels(offset.y+widgetOffset.y, scale);
+          rect.x += NSTwipsToIntPixels(offset.x+widgetOffset.x, scale);
+          rect.y += NSTwipsToIntPixels(offset.y+widgetOffset.y, scale);
         }
         
         size = frame->GetSize();
-        rect->width = NSTwipsToIntPixels(size.width, scale);
-        rect->height = NSTwipsToIntPixels(size.height, scale);
+        rect.width = NSTwipsToIntPixels(size.width, scale);
+        rect.height = NSTwipsToIntPixels(size.height, scale);
       }
     }
   }
   
-  return *rect;
+  return rect;
 }
 
 ////////////////////////////////////////////////////////////////////////
@@ -463,8 +465,8 @@ nsFormFillController::OnTextEntered(PRBool *_retval)
 
   nsCOMPtr<nsIDOMEventTarget> targ = do_QueryInterface(mFocusedInput);
 
-  PRBool noDefault;
-  targ->DispatchEvent(event, &noDefault);
+  PRBool defaultActionEnabled;
+  targ->DispatchEvent(event, &defaultActionEnabled);
 
   return NS_OK;
 }
@@ -487,6 +489,8 @@ nsFormFillController::StartSearch(const nsAString &aSearchString, const nsAStrin
   nsCOMPtr<nsIAutoCompleteMdbResult> mdbResult = do_QueryInterface(aPreviousResult);
 
   nsPasswordManager* passMgr = nsPasswordManager::GetInstance();
+  if (!passMgr)
+    return NS_ERROR_OUT_OF_MEMORY;
 
   // Only hand off a previous result to the password manager if it's
   // a password manager result (i.e. not an nsIAutoCompleteMdbResult).
@@ -497,12 +501,13 @@ nsFormFillController::StartSearch(const nsAString &aSearchString, const nsAStrin
                                    getter_AddRefs(result)))
   {
     nsFormHistory *history = nsFormHistory::GetInstance();
-    history->AutoCompleteSearch(aSearchParam,
-                                aSearchString,
-                                mdbResult,
-                                getter_AddRefs(result));
-
-    NS_RELEASE(history);
+    if (history) {
+      history->AutoCompleteSearch(aSearchParam,
+                                  aSearchString,
+                                  mdbResult,
+                                  getter_AddRefs(result));
+      NS_RELEASE(history);
+    }
   }
   NS_RELEASE(passMgr);
 
@@ -656,6 +661,49 @@ nsFormFillController::KeyPress(nsIDOMEvent* aEvent)
   return NS_OK;
 }
 
+////////////////////////////////////////////////////////////////////////
+//// nsIDOMCompositionListener
+
+NS_IMETHODIMP
+nsFormFillController::HandleStartComposition(nsIDOMEvent* aCompositionEvent)
+{
+  NS_ASSERTION(mController, "should have a controller!");
+
+  if (mController && mFocusedInput)
+    mController->HandleStartComposition();
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFormFillController::HandleEndComposition(nsIDOMEvent* aCompositionEvent)
+{
+  NS_ASSERTION(mController, "should have a controller!");
+
+  if (mController && mFocusedInput)
+    mController->HandleEndComposition();
+
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFormFillController::HandleQueryComposition(nsIDOMEvent* aCompositionEvent)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFormFillController::HandleQueryReconversion(nsIDOMEvent* aCompositionEvent)
+{
+  return NS_OK;
+}
+
+NS_IMETHODIMP
+nsFormFillController::HandleQueryCaretRect(nsIDOMEvent* aCompostionEvent)
+{
+  return NS_OK;
+}
+
 ////////////////////////////////////////////////////////////////////////
 //// nsIDOMFormListener
 
@@ -689,7 +737,7 @@ nsFormFillController::Select(nsIDOMEvent* aEvent)
 NS_IMETHODIMP
 nsFormFillController::Input(nsIDOMEvent* aEvent)
 {
-  if (mSuppressOnInput || !mController)
+  if (mSuppressOnInput || !mController || !mFocusedInput)
     return NS_OK;
 
   return mController->HandleText(PR_FALSE);
@@ -757,7 +805,7 @@ nsFormFillController::MouseClick(nsIDOMEvent* aMouseEvent)
   input->GetTextValue(value);
   if (value.Length() > 0) {
     // Show the popup with a filtered result set
-    mController->SetSearchString(NS_LITERAL_STRING(""));
+    mController->SetSearchString(EmptyString());
     mController->HandleText(PR_TRUE);
   } else {
     // Show the popup with the complete result set.  Can't use HandleText()
@@ -842,9 +890,10 @@ nsFormFillController::AddWindowListeners(nsIDOMWindow *aWindow)
     return;
 
   nsCOMPtr<nsPIDOMWindow> privateDOMWindow(do_QueryInterface(aWindow));
-  nsCOMPtr<nsIChromeEventHandler> chromeEventHandler;
+  nsIChromeEventHandler* chromeEventHandler = nsnull;
   if (privateDOMWindow)
-    privateDOMWindow->GetChromeEventHandler(getter_AddRefs(chromeEventHandler));
+    chromeEventHandler = privateDOMWindow->GetChromeEventHandler();
+
   nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(chromeEventHandler));
 
   if (!target)
@@ -873,6 +922,14 @@ nsFormFillController::AddWindowListeners(nsIDOMWindow *aWindow)
   target->AddEventListener(NS_LITERAL_STRING("unload"),
                            NS_STATIC_CAST(nsIDOMLoadListener *, this),
                            PR_TRUE);
+
+  target->AddEventListener(NS_LITERAL_STRING("compositionstart"),
+                           NS_STATIC_CAST(nsIDOMCompositionListener *, this),
+                           PR_TRUE);
+
+  target->AddEventListener(NS_LITERAL_STRING("compositionend"),
+                           NS_STATIC_CAST(nsIDOMCompositionListener *, this),
+                           PR_TRUE);
 }
 
 void
@@ -884,9 +941,10 @@ nsFormFillController::RemoveWindowListeners(nsIDOMWindow *aWindow)
   StopControllingInput();
   
   nsCOMPtr<nsPIDOMWindow> privateDOMWindow(do_QueryInterface(aWindow));
-  nsCOMPtr<nsIChromeEventHandler> chromeEventHandler;
+  nsIChromeEventHandler* chromeEventHandler;
   if (privateDOMWindow)
-    privateDOMWindow->GetChromeEventHandler(getter_AddRefs(chromeEventHandler));
+    chromeEventHandler = privateDOMWindow->GetChromeEventHandler();
+  
   nsCOMPtr<nsIDOMEventTarget> target(do_QueryInterface(chromeEventHandler));
 
   if (!target)
@@ -915,6 +973,14 @@ nsFormFillController::RemoveWindowListeners(nsIDOMWindow *aWindow)
   target->RemoveEventListener(NS_LITERAL_STRING("unload"),
                               NS_STATIC_CAST(nsIDOMLoadListener *, this),
                               PR_TRUE);
+
+  target->RemoveEventListener(NS_LITERAL_STRING("compositionstart"),
+                              NS_STATIC_CAST(nsIDOMCompositionListener *, this),
+                              PR_TRUE);
+
+  target->RemoveEventListener(NS_LITERAL_STRING("compositionend"),
+                              NS_STATIC_CAST(nsIDOMCompositionListener *, this),
+                              PR_TRUE);
 }
 
 void
@@ -1013,6 +1079,9 @@ nsFormFillController::GetWindowForDocShell(nsIDocShell *aDocShell)
 PRInt32
 nsFormFillController::GetIndexOfDocShell(nsIDocShell *aDocShell)
 {
+  if (!aDocShell)
+    return -1;
+
   // Loop through our cached docShells looking for the given docShell
   PRUint32 count;
   mDocShells->Count(&count);
@@ -1034,4 +1103,3 @@ nsFormFillController::GetIndexOfDocShell(nsIDocShell *aDocShell)
     
   return -1;
 }
-
